require 'rake/task'

class Rake::Task
  include Rake::Opt::KeywordArgs

  # Save the original invoke instance method
  alias_method "__invoke_before_keyword_args__", "invoke"

  def invoke (*args)
    if self.arg_options && args.first.is_a?(Hash)
      self.__invoke_before_keyword_args__(arg_hash_to_str(args.first))
    else
      self.__invoke_before_keyword_args__(*args)
    end
  end

  # Save the original execute instance method
  alias_method "__execute_before_keyword_args__", "execute"

  def execute (*args)
    if self.arg_options && args.first.is_a?(Hash)
      self.__execute_before_keyword_args__(Rake::TaskArguments.new(arg_names, [arg_hash_to_str(args.first)]))
    else
      self.__execute_before_keyword_args__(*args)
    end
  end

  def invoke_prerequisites(task_args, invocation_chain) # :nodoc:
    if application.options.always_multitask
      invoke_prerequisites_concurrently(task_args, invocation_chain)
    else
      prerequisite_tasks.each do |p|
        merged_args = self.verify_and_merge_args(task_args)
        p.invoke_with_call_chain(extract_prerequisite_args(task_args,p), invocation_chain)
      end
    end
  end

  def invoke_prerequisites_concurrently(task_args, invocation_chain)# :nodoc:
    futures = prerequisite_tasks.map do |p|
      merged_args = self.verify_and_merge_args(task_args)
      application.thread_pool.future(p) do |r|
        r.invoke_with_call_chain(extract_prerequisite_args(task_args,p), invocation_chain)
      end
    end
    futures.each { |f| f.value }
  end

  class << self
    # Save the original define_task module method
    alias_method "__define_task_before_keyword_args__", "define_task"
    def define_task(*args, &block)
      if Rake.application.last_args
        return define_parameterized_task(*args, &block)
      else
        return self.__define_task_before_keyword_args__(*args, &block)
      end
    end
  end

end
